/*
 *          File: HartLib.h
 *                Some classes for the definition of HART constants.
 *
 *        Author: Walter Borst
 *
 *        E-Mail: info@borst-automation.de
 *          Home: https://www.borst-automation.de
 *
 * No Warranties: https://www.borst-automation.com/legal/warranty-disclaimer
 *
 * Copyright 2006-2024 Walter Borst, Cuxhaven, Germany
 */

#ifndef __hartlib_h__
#define __hartlib_h__

#include "OSAL.h"
#include "WbHartUser.h"

class CHart
{
public:
    class CDelimiter
    {
    public:
        static const TY_Byte BACK = 1;
        static const TY_Byte STX = 2;
        static const TY_Byte ACK = 6;
    };

    class CFrameType
    {
    public:
        static const TY_Byte BACK = 0;
        static const TY_Byte STX = 1;
        static const TY_Byte ACK = 2;
        static const TY_Byte OBACK = 3;
        static const TY_Byte OSTX = 4;
        static const TY_Byte OACK = 5;
    };

    class CLimit
    {
    public:
        static const TY_Byte MAX_NUM_PREAMBLES = 20;
    };

    class CSize
    {
    public:
        static const TY_Byte NUM_ADDR_BYTES = 5;
    };

    class CMask
    {
    public:
        static const TY_Byte     DELIMITER = 0x07;
        static const TY_Byte     ADDR_TYPE = 0x80;
        static const TY_Byte     HIGH_ADDR = 0x3f;
        static const TY_Byte NUM_EXP_BYTES = 0x60;
    };

    class CAddrType
    {
    public:
        static const TY_Byte SHORT = 0;
        static const TY_Byte LONG = 1;
    };

    class CFlags
    {
    public:
        static const TY_Byte LONG_ADDR = 0x80;
        static const TY_Byte PRIM_MASTER = 0x80;
        static const TY_Byte EXT_CMD_FLAG = 31;
    };

    class CAddrMode
    {
    public:
        static const TY_Byte LONG_ADDR = 0;
        static const TY_Byte SHORT_ADDR = 1;
    };
};
#endif // __hartlib_h__