/*
 *          File: WbHartM_Typedefs.h
 *                This file contains type definitions which are used
 *                in all modules in the Hart master kernel.
 *
 *        Author: Walter Borst
 *
 *        E-Mail: info@borst-automation.de
 *          Home: https://www.borst-automation.de
 *
 * No Warranties: https://www.borst-automation.com/legal/warranty-disclaimer
 *
 * Copyright 2006-2024 Walter Borst, Cuxhaven, Germany
 */

// Once
#ifndef __wbhartm_typedefs_h__
#define __wbhartm_typedefs_h__

    // Type definitions
typedef unsigned char      TY_Byte;
typedef unsigned short     TY_Word;
typedef unsigned int       TY_DWord;
typedef int                TY_Int32;
typedef unsigned long long TY_UInt64;
typedef float              TY_Float;
typedef double             TY_DFloat;
typedef TY_Word            WRD_Handle;
typedef TY_Word            SRV_Handle;

// Enum classes 
enum class EN_SRV_Result : TY_Byte {
    EMPTY = 0,
    NO_DEV_RESP = 1,
    COMM_ERR = 2,
    INVALID_HANDLE = 3,
    IN_PROGRESS = 4,
    SUCCESSFUL = 5,
    RESOURCE_ERROR = 6,
    TOO_FEW_DATA_BYTES = 7,
    OBSOLETE = 8
};
enum class EN_Endian : TY_Byte
{
    MSB_First = 0, // Big endian (Hart standard)
    LSB_First = 1  // Little endian
};
enum class EN_Bool : TY_Byte
{
    FALSE8 = 0,
    TRUE8 = 1
};
enum class EN_Bit : TY_Byte
{
    CLEAR8 = 0,
    SET8 = 1
};
enum class EN_Error : TY_Byte
{
    NONE = 0,
    ERR = 1
};
enum class EN_Wait : TY_Byte
{
    NO_WAIT = 0,
    WAIT = 1
};
enum class EN_CommType : TY_Byte
{
    NONE = 0,
    UART = 1,
    TCP_IP = 2
};

enum class EN_Master : TY_Byte
{
    PRIMARY = 0,
    SECONDARY = 1
};

// Constants
static const TY_Word INVALID_WRD_HANDLE = (TY_Word)0xFFFF;
static const TY_Word INVALID_SRV_HANDLE = (TY_Word)0xFFFF;

#endif // __wbhartm_typedefs_h__