/*
 *          File: BaHartMaster-7.6.cpp
 *                The implementation for the calls to the Windows DLL is
 *                located here. In practice, it is just a shell through
 *                which the functions in the CUartMaster module are called.
 *                See also HartM_UartIface.cpp/h.
 *
 *        Author: Walter Borst
 *
 *        E-Mail: info@borst-automation.de
 *          Home: https://www.borst-automation.de
 *
 * No Warranties: https://www.borst-automation.com/legal/warranty-disclaimer
 *
 * Copyright 2006-2024 Walter Borst, Cuxhaven, Germany
 */

#include "BaHartMaster-7.6.h"
#include "HartM_UartIface.h"

// Main entry point (Windows standard)
BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
                     )
{
    switch (ul_reason_for_call)
    {
    case DLL_PROCESS_ATTACH:
    case DLL_THREAD_ATTACH:
    case DLL_THREAD_DETACH:
    case DLL_PROCESS_DETACH:
        break;
    }
    return TRUE;
}

// Channel handling

HARTDLL_API EN_Bool WINAPI BAHAMA_OpenChannel(TY_Word port_number_)
{
    return CUartMaster::OpenChannel(port_number_, EN_CommType::UART);
}

HARTDLL_API void WINAPI BAHAMA_CloseChannel()
{
    CUartMaster::CloseChannel();
}

// Configuration

HARTDLL_API void WINAPI BAHAMA_GetConfiguration(TY_Configuration* config_)
{
    CUartMaster::GetConfiguration(config_);
}

HARTDLL_API void WINAPI BAHAMA_SetConfiguration(TY_Configuration* config_)
{
    CUartMaster::SetConfiguration(config_);
}

// Connection

HARTDLL_API SRV_Handle WINAPI BAHAMA_ConnectByAddr(
    TY_Byte address_,
    EN_Wait qos_,
    TY_Byte num_retries_)
{
    return CUartMaster::ConnectByAddr(address_, qos_, num_retries_);
}

HARTDLL_API void WINAPI BAHAMA_FetchConnection(SRV_Handle service_, TY_Connection* connection_)
{
    CUartMaster::FetchConnection(service_, connection_);
}

// Commands

HARTDLL_API SRV_Handle WINAPI BAHAMA_DoCommand(
    TY_Byte  command_,
    EN_Wait  qos_,
    TY_Byte* data_ref_,
    TY_Byte  data_len_,
    TY_Byte* bytes_of_unique_id_)
{
    return CUartMaster::LaunchCommand(command_, qos_, data_ref_, data_len_, bytes_of_unique_id_);
}

HARTDLL_API WRD_Handle WINAPI BAHAMA_DoExtCmd(
    TY_Word  command_,
    EN_Wait  qos_,
    TY_Byte* data_ref_,
    TY_Byte  data_len_,
    TY_Byte* bytes_of_unique_id_)
{
    return CUartMaster::LaunchExtCommand(command_, qos_, data_ref_, data_len_, bytes_of_unique_id_);
}

// Service handling

HARTDLL_API EN_Bool WINAPI BAHAMA_IsServiceCompleted(SRV_Handle service_)
{
    return CUartMaster::IsServiceCompleted(service_);
}

HARTDLL_API void WINAPI BAHAMA_FetchConfirmation(SRV_Handle service_,
    TY_Confirmation* conf_data_)
{
    CUartMaster::FetchConfirmation(service_, conf_data_);
}

// Encoding
HARTDLL_API void WINAPI BAHAMA_PutInt8(TY_Byte data_, TY_Byte offset_,
    TY_Byte* data_ref_)
{
    CUartMaster::PutInt8(data_, offset_, data_ref_);
}

HARTDLL_API void WINAPI BAHAMA_PutInt16(TY_Word data_, TY_Byte offset_,
    TY_Byte* data_ref_,
    EN_Endian endian_)
{
    CUartMaster::PutInt16(data_, offset_, data_ref_, endian_);
}

HARTDLL_API void WINAPI BAHAMA_PutInt24(TY_DWord data_, TY_Byte offset_,
    TY_Byte* data_ref_,
    EN_Endian endian_)
{
    CUartMaster::PutInt24(data_, offset_, data_ref_, endian_);
}

HARTDLL_API void WINAPI BAHAMA_PutInt32(TY_DWord data_, TY_Byte offset_,
    TY_Byte* data_ref_,
    EN_Endian endian_)
{
    CUartMaster::PutInt32(data_, offset_, data_ref_, endian_);
}

HARTDLL_API void WINAPI BAHAMA_PutInt64(TY_UInt64 data_, TY_Byte offset_,
    TY_Byte* data_ref_,
    EN_Endian  endian_)
{
    CUartMaster::PutInt64(data_, offset_, data_ref_, endian_);
}

HARTDLL_API void WINAPI BAHAMA_PutFloat(TY_Float data_, TY_Byte offset_,
    TY_Byte* data_ref_,
    EN_Endian  endian_)
{
    CUartMaster::PutFloat(data_, offset_, data_ref_, endian_);
}

HARTDLL_API void WINAPI BAHAMA_PutDouble(TY_DFloat data_, TY_Byte offset_,
    TY_Byte* data_ref_,
    EN_Endian  endian_)
{
    CUartMaster::PutDFloat(data_, offset_, data_ref_, endian_);
}

HARTDLL_API void WINAPI BAHAMA_PutPackedASCII(TY_Byte* sb_,
    TY_Byte       len_,
    TY_Byte    offset_,
    TY_Byte* data_ref_)
{
    CUartMaster::PutPackedASCII(sb_, len_, offset_, data_ref_);
}

HARTDLL_API void WINAPI BAHAMA_PutOctets(TY_Byte* stream_ref_,
    TY_Byte stream_len_,
    TY_Byte offset_,
    TY_Byte* data_ref_)
{
    CUartMaster::PutOctets(stream_ref_, stream_len_, offset_, data_ref_);
}

HARTDLL_API void WINAPI BAHAMA_PutString(TY_Byte* string_ref_,
    TY_Byte string_max_len_,
    TY_Byte offset_,
    TY_Byte* data_ref_)
{
    CUartMaster::PutString(string_ref_, string_max_len_, offset_, data_ref_);
}


HARTDLL_API TY_Byte WINAPI BAHAMA_PickInt8(TY_Byte offset_,
    TY_Byte* data_ref_)
{
    return CUartMaster::PickInt8(offset_, data_ref_);
}

HARTDLL_API TY_Word WINAPI BAHAMA_PickInt16(TY_Byte offset_,
    TY_Byte* data_ref_,
    EN_Endian endian_)
{
    return CUartMaster::PickInt16(offset_, data_ref_, endian_);
}

HARTDLL_API TY_DWord WINAPI BAHAMA_PickInt24(TY_Byte offset_,
    TY_Byte* data_ref_,
    EN_Endian endian_)
{
    return CUartMaster::PickInt24(offset_, data_ref_, endian_);
}

HARTDLL_API TY_DWord WINAPI BAHAMA_PickInt32(TY_Byte offset_,
    TY_Byte* data_ref_,
    EN_Endian endian_)
{
    return CUartMaster::PickInt32(offset_, data_ref_, endian_);
}

HARTDLL_API TY_UInt64 WINAPI BAHAMA_PickInt64(TY_Byte offset_,
    TY_Byte* data_ref_,
    EN_Endian endian_)
{
    return CUartMaster::PickInt64(offset_, data_ref_, endian_);
}

HARTDLL_API TY_Float WINAPI BAHAMA_PickFloat(TY_Byte offset_,
    TY_Byte* data_ref_,
    EN_Endian endian_)
{
    return CUartMaster::PickFloat(offset_, data_ref_, endian_);
}

HARTDLL_API TY_DFloat WINAPI BAHAMA_PickDouble(TY_Byte offset_,
    TY_Byte* data_ref_,
    EN_Endian endian_)
{
    return CUartMaster::PickDFloat(offset_, data_ref_, endian_);
}

HARTDLL_API void WINAPI BAHAMA_PickPackedASCII(TY_Byte* string_ref_,
    TY_Byte string_max_len_,
    TY_Byte offset_,
    TY_Byte* data_ref_)
{
    CUartMaster::PickPackedASCII(string_ref_, string_max_len_, offset_, data_ref_);
}

HARTDLL_API void WINAPI BAHAMA_PickOctets(TY_Byte* stream_ref_,
    TY_Byte stream_len_,
    TY_Byte offset_,
    TY_Byte* data_ref_)
{
    CUartMaster::PickOctets(stream_ref_, stream_len_, offset_, data_ref_);
}

HARTDLL_API void WINAPI BAHAMA_PickString(TY_Byte* string_ref_,
    TY_Byte string_max_len_,
    TY_Byte offset_,
    TY_Byte* data_ref_)
{
    CUartMaster::PickString(string_ref_, string_max_len_, offset_, data_ref_);
}
