/*
 *          File: Settings.cs (CSettings)
 *                A .NET component is used to store and read the user 
 *                settings. Nevertheless, the individual settings must 
 *                be assigned to specific functionalities.
 *
 *        Author: Walter Borst
 *
 *        E-Mail: info@borst-automation.de
 *          Home: https://www.borst-automation.de
 *
 * No Warranties: https://www.borst-automation.com/legal/warranty-disclaimer
 *
 * Copyright 2006-2024 Walter Borst, Cuxhaven, Germany
 */

#region Namespaces
using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Text;
using System.Globalization;
using BaTestCppMaster.Properties;
using System.Drawing;
#endregion Namespaces

namespace BaTestCppMaster
{
    internal static class CSettings
    {
        #region Public Data
        // Tab File

        // Tab Start
        internal static bool ViewFrameNumbers = false;
        internal static bool ViewPreambles = false;
        internal static bool ViewAddressing = false;
        internal static bool ViewTiming = false;
        internal static bool ViewStatusBinary = false;
        internal static bool ViewDecodedData = false;
        internal static byte ComPort = 0;
        internal static byte PollAddress = 0;
        internal static byte BaudRate = 1;
        internal static byte NumPreambles = 5;
        internal static byte InitialMasterRole = 0;
        internal static bool OptionsLsbFirst = false;

        // Tab Commands
        internal static byte CustomCmd = 0;
        internal static string CustomCmdData = string.Empty;
        internal static string TagNameShort = string.Empty;
        internal static string TagNameLong = string.Empty;
        internal static string Descriptor = string.Empty;
        internal static byte Day = 1;
        internal static byte Month = 1;
        internal static byte Year = 107;
        internal static ushort ExtCmd = 0;
        internal static string ExtCmdData = string.Empty;
        internal static byte RangeUnitIdx = 0;
        internal static float UpperRange = 100.0f;
        internal static float LowerRange = 0.0f;

        // Tab Options
        internal static bool OptionsTopMost = false;
        internal static byte NumRetries = 2;
        internal static byte RetryIfBusy = 0;

        // Tab Options (Colors)
        internal static Color MonColLinNum = Color.Black;
        internal static Color MonColTime = Color.SaddleBrown;
        internal static Color MonColHeader = Color.Blue;
        internal static Color MonColData = Color.Black;
        internal static Color MonColJabber = Color.DarkViolet;
        internal static Color MonColGarbage = Color.Purple;
        internal static Color MonColError = Color.Red;
        internal static Color MonColTrigger = Color.Gold;
        internal static Color MonColTriggerBk = Color.PowderBlue;
        internal static Color MonColTrigSel = Color.RoyalBlue;
        internal static Color MonColDataSel = Color.DimGray;
        internal static Color MonColBack = Color.Bisque;
        internal static Color MonColPrimMaster = Color.DarkBlue;
        internal static Color MonColScndMaster = Color.SaddleBrown;
        internal static Color MonColWrongData = Color.DarkRed;
        internal static Color MonColRespIndication = Color.DarkKhaki;
        internal static Color StatBarColBkNotRec = Color.LightGreen;
        internal static Color StatBarColBkRecording = Color.LightCoral;
        internal static Color ButColBkHot = Color.PowderBlue;
        internal static int ColorSet = 0;

        // Run time control
        internal static bool ModeMonitorFrames = false;

        // Main Form Appearance
        internal static int MainFormTop = 0;
        internal static int MainFormLeft = 0;
        internal static int MainFormHeight = 400;
        internal static int MainFormWidth = 600;
        #endregion Public Data  

        #region User Colors Data
        private static Color MonColLinNumUser;
        private static Color MonColTimeUser;
        private static Color MonColHeaderUser;
        private static Color MonColDataUser;
        private static Color MonColJabberUser;
        private static Color MonColGarbageUser;
        private static Color MonColErrorUser;
        private static Color MonColTriggerUser;
        private static Color MonColTriggerBkUser;
        private static Color MonColTrigSelUser;
        private static Color MonColDataSelUser;
        private static Color MonColBackUser;
        private static Color MonColPrimMasterUser;
        private static Color MonColScndMasterUser;
        private static Color MonColWrongDataUser;
        private static Color MonColBusyIndicationUser;
        private static Color StatBarColBkNotRecUser;
        private static Color StatBarColBkRecordingUser;
        private static Color ButColBkHotUser;
        #endregion User Colors Data

        #region Public Methods
        internal static void SaveUserColors()
        {
            MonColLinNumUser = MonColLinNum;
            MonColTimeUser = MonColTime;
            MonColHeaderUser = MonColHeader;
            MonColDataUser = MonColData;
            MonColJabberUser = MonColJabber;
            MonColGarbageUser = MonColGarbage;
            MonColErrorUser = MonColError;
            MonColTriggerUser = MonColTrigger;
            MonColTriggerBkUser = MonColTriggerBk;
            MonColTrigSelUser = MonColTrigSel;
            MonColDataSelUser = MonColDataSel;
            MonColBackUser = MonColBack;
            MonColPrimMasterUser = MonColPrimMaster;
            MonColScndMasterUser = MonColScndMaster;
            MonColWrongDataUser = MonColWrongData;
            MonColBusyIndicationUser = MonColRespIndication;
            StatBarColBkNotRecUser = StatBarColBkNotRec;
            StatBarColBkRecordingUser = StatBarColBkRecording;
            ButColBkHotUser = ButColBkHot;
        }

        internal static void GetColorSet1()
        {
            MonColBack = Color.FromArgb(220, 220, 220);
            MonColPrimMaster = Color.FromArgb(0, 0, 255);
            MonColScndMaster = Color.FromArgb(0, 102, 51);
            MonColData = Color.FromArgb(0, 0, 0);
            MonColWrongData = Color.FromArgb(139, 0, 0);
            MonColHeader = Color.FromArgb(0, 0, 255);
            MonColTime = Color.FromArgb(139, 69, 19);
            MonColLinNum = Color.FromArgb(0, 0, 0);
            MonColJabber = Color.FromArgb(148, 0, 211);
            MonColGarbage = Color.FromArgb(185, 92, 0);
            MonColError = Color.FromArgb(255, 0, 0);
            MonColRespIndication = Color.FromArgb(0, 128, 0);
            MonColDataSel = Color.FromArgb(105, 105, 105);
            MonColTrigger = Color.FromArgb(255, 215, 0);
            MonColTriggerBk = Color.FromArgb(176, 224, 230);
            MonColTrigSel = Color.FromArgb(65, 105, 225);
            StatBarColBkNotRec = Color.FromArgb(220, 220, 220);
            StatBarColBkRecording = Color.FromArgb(208, 181, 159);
            ButColBkHot = Color.FromArgb(176, 224, 230);
        }

        internal static void GetColorSet2()
        {
            MonColBack = Color.FromArgb(0, 0, 0);
            MonColPrimMaster = Color.FromArgb(0, 255, 255);
            MonColScndMaster = Color.FromArgb(128, 255, 128);
            MonColData = Color.FromArgb(255, 255, 255);
            MonColWrongData = Color.FromArgb(255, 128, 64);
            MonColHeader = Color.FromArgb(255, 128, 255);
            MonColTime = Color.FromArgb(255, 255, 0);
            MonColLinNum = Color.FromArgb(224,224,224);
            MonColJabber = Color.FromArgb(128, 255, 255); ;
            MonColGarbage = Color.FromArgb(255, 255, 128);
            MonColError = Color.FromArgb(255, 196, 196);
            MonColRespIndication = Color.FromArgb(128, 255, 128);
            MonColDataSel = Color.FromArgb(176, 176, 176);
            MonColTrigger = Color.FromArgb(255, 215, 0);
            MonColTriggerBk = Color.FromArgb(176, 224, 230);
            MonColTrigSel = Color.FromArgb(255, 248, 220);
            StatBarColBkNotRec = Color.FromArgb(144, 238, 144);
            StatBarColBkRecording = Color.FromArgb(240, 128, 128);
            ButColBkHot = Color.FromArgb(176,224,230);
        }

        internal static void GetColorSetUser()
        {
            MonColLinNum = MonColLinNumUser;
            MonColTime = MonColTimeUser;
            MonColHeader = MonColHeaderUser;
            MonColData = MonColDataUser;
            MonColJabber = MonColJabberUser;
            MonColGarbage = MonColGarbageUser;
            MonColError = MonColErrorUser;
            MonColTrigger = MonColTriggerUser;
            MonColTriggerBk = MonColTriggerBkUser;
            MonColTrigSel = MonColTrigSelUser;
            MonColDataSel = MonColDataSelUser;
            MonColBack = MonColBackUser;
            MonColPrimMaster = MonColPrimMasterUser;
            MonColScndMaster = MonColScndMasterUser;
            MonColWrongData = MonColWrongDataUser;
            MonColRespIndication = MonColBusyIndicationUser;
            StatBarColBkNotRec = StatBarColBkNotRecUser;
            StatBarColBkRecording = StatBarColBkRecordingUser;
            ButColBkHot = ButColBkHotUser;
        }

        internal static void Fetch()
        {
            // Tab Start
            ViewFrameNumbers = Settings.Default.ViewFrameNumbers;
            ViewPreambles = Settings.Default.ViewPreambles;
            ViewAddressing = Settings.Default.ViewAddressing;
            ViewTiming = Settings.Default.ViewTiming;
            ViewStatusBinary = Settings.Default.ViewStatusBinary;
            ViewDecodedData = Settings.Default.ViewDecodedData;
            ComPort = Settings.Default.ComPort;
            PollAddress = Settings.Default.PollAddress;
            BaudRate = Settings.Default.BaudRate;
            NumPreambles = Settings.Default.NumPreambles;
            InitialMasterRole = Settings.Default.InitialMasterRole;
            OptionsLsbFirst = Settings.Default.OptionsLsbFirst;

            // Tab Commands
            CustomCmd = Settings.Default.CustomCmd;
            CustomCmdData = Settings.Default.CustomCmdData;
            Descriptor = Settings.Default.Descriptor;
            CSettings.Day = Settings.Default.Day;
            CSettings.Month = Settings.Default.Month;
            CSettings.Year = Settings.Default.Year;
            CSettings.ExtCmd = Settings.Default.ExtCmd;
            CSettings.ExtCmdData = Settings.Default.ExtCmdData;
            CSettings.RangeUnitIdx = Settings.Default.RangeUnitIdx;
            CSettings.UpperRange = Settings.Default.UpperRange;
            CSettings.LowerRange = Settings.Default.LowerRange;

            // Tab Options
            OptionsTopMost = Settings.Default.OptionsTopMost;
            NumRetries = Settings.Default.NumRetries;
            RetryIfBusy = Settings.Default.RetryIfBusy;

            // Tab Options (Colors)
            MonColLinNum = Settings.Default.MonColLinNum;
            MonColTime = Settings.Default.MonColTime;
            MonColHeader = Settings.Default.MonColHeader;
            MonColData = Settings.Default.MonColData;
            MonColJabber = Settings.Default.MonColJabber;
            MonColGarbage = Settings.Default.MonColGarbage;
            MonColError = Settings.Default.MonColError;
            MonColDataSel = Settings.Default.MonColDataSel;
            MonColBack = Settings.Default.MonColBack;
            MonColPrimMaster = Settings.Default.MonColPrimMaster;
            MonColScndMaster = Settings.Default.MonColScndMaster;
            MonColWrongData = Settings.Default.MonColWrongData;
            MonColRespIndication = Settings.Default.MonColBusyIndication;
            StatBarColBkNotRec = Settings.Default.StatBarColBkNotRec;
            StatBarColBkRecording = Settings.Default.StatBarColBkRecording;
            ButColBkHot = Settings.Default.ButColBkHot;
            ColorSet = Settings.Default.ColorSet;

            // Run time control
            ModeMonitorFrames = Settings.Default.ModeMonitorFrames;

            // Main Form Appearance
            MainFormTop = Settings.Default.MainFormTop;
            MainFormLeft = Settings.Default.MainFormLeft;
            MainFormHeight = Settings.Default.MainFormHeight;
            MainFormWidth = Settings.Default.MainFormWidth;
        }

        internal static void Put()
        {
            // Tab File

            // Tab Start
            Settings.Default.ViewFrameNumbers = ViewFrameNumbers;
            Settings.Default.ViewPreambles = ViewPreambles;
            Settings.Default.ViewAddressing = ViewAddressing;
            Settings.Default.ViewTiming = ViewTiming;
            Settings.Default.ViewStatusBinary = ViewStatusBinary;
            Settings.Default.ViewDecodedData = ViewDecodedData;
            Settings.Default.ComPort = ComPort;
            Settings.Default.PollAddress = PollAddress;
            Settings.Default.BaudRate = BaudRate;
            Settings.Default.NumPreambles = NumPreambles;
            Settings.Default.InitialMasterRole = InitialMasterRole;
            Settings.Default.OptionsLsbFirst = OptionsLsbFirst;

            // Tab Commands
            Settings.Default.CustomCmd = CustomCmd;
            Settings.Default.CustomCmdData = CustomCmdData;
            Settings.Default.Descriptor = CSettings.Descriptor;
            Settings.Default.Day = CSettings.Day;
            Settings.Default.Month = CSettings.Month;
            Settings.Default.Year = CSettings.Year;
            Settings.Default.ExtCmd = CSettings.ExtCmd;
            Settings.Default.ExtCmdData = CSettings.ExtCmdData;
            Settings.Default.RangeUnitIdx = CSettings.RangeUnitIdx;
            Settings.Default.UpperRange = CSettings.UpperRange;
            Settings.Default.LowerRange = CSettings.LowerRange;

            // Tab Options
            Settings.Default.OptionsTopMost = OptionsTopMost;
            Settings.Default.NumRetries = NumRetries;
            Settings.Default.RetryIfBusy = RetryIfBusy;

            // Tab Options (Colors)
            Settings.Default.MonColLinNum = MonColLinNum;
            Settings.Default.MonColTime = MonColTime;
            Settings.Default.MonColHeader = MonColHeader;
            Settings.Default.MonColData = MonColData;
            Settings.Default.MonColJabber = MonColJabber;
            Settings.Default.MonColGarbage = MonColGarbage;
            Settings.Default.MonColError = MonColError;
            Settings.Default.MonColDataSel = MonColDataSel;
            Settings.Default.MonColBack = MonColBack;
            Settings.Default.MonColPrimMaster = MonColPrimMaster;
            Settings.Default.MonColScndMaster = MonColScndMaster;
            Settings.Default.MonColWrongData = MonColWrongData;
            Settings.Default.MonColBusyIndication = MonColRespIndication;
            Settings.Default.StatBarColBkNotRec = StatBarColBkNotRec;
            Settings.Default.StatBarColBkRecording = StatBarColBkRecording;
            Settings.Default.ButColBkHot = ButColBkHot;
            Settings.Default.ColorSet = ColorSet;

            // Run time control
            Settings.Default.ModeMonitorFrames = ModeMonitorFrames;

            // Main Form Appearance
            Settings.Default.MainFormTop = MainFormTop;
            Settings.Default.MainFormLeft = MainFormLeft;
            Settings.Default.MainFormHeight = MainFormHeight;
            Settings.Default.MainFormWidth = MainFormWidth;
        }
        #endregion Public Methods
    }
}

